from django.db import models
from django.contrib.auth.models import User
from django.contrib.contenttypes.models import ContentType, ContentTypeManager 
from django.contrib.contenttypes import generic 
from datetime import datetime


class RoomManager(models.Manager):

    def create(self, user, roomName, comment, password, public):
        r = self.model(owner=user, roomName=roomName, comment=comment, password=password, public=public)
        r.save()
        return r
        
    def get_for_object(self, roomName):
        '''Try to get a room related to the object passed.'''
	return self.get(roomName = roomName)

class Room(models.Model):
    '''Representation of a generic chat room'''
    owner = models.ForeignKey(User, related_name='owner', blank=True, null=True)
    created = models.DateTimeField(default=datetime.now())
    roomName = models.CharField(max_length=255, blank=False)
    comment = models.CharField(max_length=255, blank=True, null=True)
    public = models.BooleanField(default=False)
    password = models.CharField(max_length=255)
    deleted = models.BooleanField(default=False)
    objects = RoomManager() 
    
    def __add_message(self, type, sender, message=None):
        '''Generic function for adding a message to the chat room'''
        m = Message(room=self, type=type, author=sender, message=message)
        m.save()
        return m
    
    def say(self, sender, message):
        '''Say something in to the chat room'''
        return self.__add_message('m', sender, message)
    
    def join(self, user):
        '''A user has joined'''
        return self.__add_message('j', user)
    
    def leave(self, user):
        '''A user has leaved'''
        return self.__add_message('l', user)
    
    def messages(self, after_pk=None, after_date=None):
        '''List messages, after the given id or date'''
        m = Message.objects.filter(room=self)
        if after_pk:
            m = m.filter(pk__gt=after_pk)
        if after_date:
            m = m.filter(timestamp__gt=after_date)
        return m.order_by('timestamp')
    
    def last_message_id(self):
        '''Return last message sent to room'''
        m = Message.objects.filter(room=self).order_by('-pk')
        if m:
            return m[0].id
        else:
            return 0

    def last_message_time(self):
	m = Message.objects.filter(room=self).order_by('-timestamp')
	if m:
	    return m[0].timestamp
	else:
	    return datetime.now()
    
    def __unicode__(self):
        return '%s' % (self.roomName)
    
#    class Meta:
#        unique_together = (("content_type", "object_id"),)


MESSAGE_TYPE_CHOICES = (
    ('s','system'),
    ('a','action'),
    ('m', 'message'),
    ('j','join'),
    ('l','leave'),
    ('n','notification')
)


class Message(models.Model):
    '''A message that belongs to a chat room'''
    room = models.ForeignKey(Room)
    type = models.CharField(max_length=1, choices=MESSAGE_TYPE_CHOICES)
    author = models.ForeignKey(User, related_name='author', blank=True, null=True)
    message = models.CharField(max_length=255, blank=True, null=True)
    timestamp = models.DateTimeField(auto_now=True)
    
    def __unicode__(self):
        '''Each message type has a special representation, return that representation.
        This will also be translator AKA i18l friendly.''' 
        if self.type == 's':
            return u'SYSTEM: %s' % self.message
        if self.type == 'n':
            return u'NOTIFICATION: %s' % self.message
        elif self.type == 'j':
            return 'JOIN: %s' % self.author
        elif self.type == 'l':
            return 'LEAVE: %s' % self.author
        elif self.type == 'a':
            return 'ACTION: %s > %s' % (self.author, self.message)
        return self.message
 
class RoomUser(models.Model):
	rname = models.CharField(max_length=255)
	uname = models.CharField(max_length=255)
	talk_permission = models.BooleanField(default=True)
	blind = models.BooleanField(default=True)
	inroom = models.BooleanField(default=False)
	
	def __str__(self):
        	return 'ROOM_USER: %s > %s' % (self.rname, self.uname)
        	
class RoomFile(models.Model):
	rname = models.ForeignKey(Room)
	uname = models.ForeignKey(User)
	myfile = models.FileField(upload_to='files')
        
class Poll(models.Model):
	subject = models.CharField(max_length=255, blank=False, null=False)
	created_on = models.DateTimeField(auto_now=True)
	creater = models.ForeignKey(User, related_name='creater', blank=True, null=True)
	
	def __str__(self):
        	return '%s' %self.subject
        	
class Choice(models.Model):
	poll = models.ForeignKey(Poll, related_name='choice_owner', blank=True, null=True)
	choice = models.CharField(max_length=255, blank=False, null=False)
	votes = models.IntegerField(default=0)
	
	def __str__(self):
        	return '%s' %self.choice
